@@ -3,7 +3,6 @@
 Functions for decoding data of various types including field tables and arrays
 
 """
-
 __author__ = 'Gavin M. Roy'
 __email__ = 'gavinmroy@gmail.com'
 __since__ = '2011-03-29'
@@ -150,7 +149,8 @@ def short_int(value):
 def short_str(value):
     """Decode a string value
 
-    :param str value: Value to decode
+    :param value: Value to decode
+    :type value: str or bytes
     :return tuple: bytes used, unicode|str
     :raises: ValueError
 
