@@ -3,7 +3,6 @@
 Functions for encoding data of various types including field tables and arrays
 
 """
-
 __author__ = 'Gavin M. Roy'
 __email__ = 'gavinmroy@gmail.com'
 __since__ = '2011-03-29'
@@ -21,7 +20,7 @@ def bit(value, byte, position):
     :param int value: Value to decode
     :param int byte: The byte to apply the value to
     :param int position: The position in the byte to set the bit on
-    :rtype: tuple of bytes used and a bool value
+    :rtype: byte
 
     """
     return byte | (value << position)
@@ -200,7 +199,7 @@ def field_table(value):
 
     # Iterate through all of the keys and encode the data into a table
     data = list()
-    for key in value:
+    for key in sorted(value.keys()):
         # Append the field header / delimiter
         data.append(struct.pack('B', len(key)))
         data.append(key)
