@@ -1,6 +1,6 @@
-"""Manage the marshaling and demarshaling of AMQP frames
+"""Manage the marshaling and unmarshaling of AMQP frames
 
-demarshal will turn a raw AMQP byte stream into the appropriate AMQP objects
+unmarshal will turn a raw AMQP byte stream into the appropriate AMQP objects
 from the specification file.
 
 marshal will take an object created from the specification file and turn it
@@ -16,14 +16,20 @@ from pamqp import exceptions
 from pamqp import heartbeat
 from pamqp import header
 from pamqp import specification
+from pamqp import PYTHON3
 
-DEMARSHALLING_FAILURE = 0, 0, None
+AMQP = b'AMQP' if PYTHON3 else 'AMQP'
 FRAME_HEADER_SIZE = 7
 FRAME_END_CHAR = chr(specification.FRAME_END)
+DECODE_FRAME_END_CHAR = FRAME_END_CHAR
+if PYTHON3:
+    FRAME_END_CHAR = bytes(FRAME_END_CHAR, 'latin-1')
+    DECODE_FRAME_END_CHAR = specification.FRAME_END
 LOGGER = logging.getLogger(__name__)
+UNMARSHAL_FAILURE = 0, 0, None
 
 
-def demarshal(data_in):
+def unmarshal(data_in):
     """Takes in binary data and maps builds the appropriate frame type,
     returning a frame object.
 
@@ -34,47 +40,56 @@ def demarshal(data_in):
     """
     # Look to see if it's a protocol header frame
     try:
-        frame_value = _demarshal_protocol_header_frame(data_in)
+        frame_value = _unmarshal_protocol_header_frame(data_in)
         if frame_value:
             return 8, 0, frame_value
     except ValueError as error:
-        raise exceptions.DemarshalingException(header.ProtocolHeader, error)
+        raise exceptions.UnmarshalingException(header.ProtocolHeader, error)
 
     # Decode the low level frame and break it into parts
     try:
         frame_type, channel_id, frame_size = _frame_parts(data_in)
-        byte_count = FRAME_HEADER_SIZE + frame_size + 1
 
-        if data_in[byte_count - 1] != FRAME_END_CHAR:
-            raise exceptions.DemarshalingException('Unknown', 'Last byte error')
+        # Heartbeats do not have frame length indicators
+        if frame_type == specification.FRAME_HEARTBEAT and frame_size == 0:
+            return 8, channel_id, heartbeat.Heartbeat()
+
+        if not frame_size:
+            raise exceptions.UnmarshalingException('Unknown', 'No frame size')
+
+        byte_count = FRAME_HEADER_SIZE + frame_size + 1
+        if byte_count > len(data_in):
+            raise exceptions.UnmarshalingException('Unknown',
+                                                   'Not all data received')
+        if data_in[byte_count - 1] != DECODE_FRAME_END_CHAR:
+            raise exceptions.UnmarshalingException('Unknown', 'Last byte error')
+
         frame_data = data_in[FRAME_HEADER_SIZE:byte_count - 1]
-    except ValueError, error:
-        raise exceptions.DemarshalingException('Unknown', error)
+
+    except ValueError as error:
+        raise exceptions.UnmarshalingException('Unknown', error)
 
     # Decode a method frame
     if frame_type == specification.FRAME_METHOD:
-        return byte_count, channel_id, _demarshal_method_frame(frame_data)
+        return byte_count, channel_id, _unmarshal_method_frame(frame_data)
 
     # Decode a header frame
     elif frame_type == specification.FRAME_HEADER:
-        return byte_count, channel_id, _demarshal_header_frame(frame_data)
+        return byte_count, channel_id, _unmarshal_header_frame(frame_data)
 
     # Decode a body frame
     elif frame_type == specification.FRAME_BODY:
-        return byte_count, channel_id, _demarshal_body_frame(frame_data)
+        return byte_count, channel_id, _unmarshal_body_frame(frame_data)
 
-    # Decode a heartbeat frame
-    elif frame_type == specification.FRAME_HEARTBEAT:
-        return byte_count, channel_id, heartbeat.Heartbeat()
-
-    exceptions.DemarshalingException('Unknown',
+    exceptions.UnmarshalingException('Unknown',
                                      'Unknown frame type: %i' % frame_type)
 
 
 def marshal(frame_value, channel_id):
     """Marshal a frame to be sent over the wire.
 
-    :param pamqp.specification.Frame frame_value: The frame object to marshal
+    :param frame_value: The frame object to marshal
+    :type frame_value: pamqp.specification.Frame or pamqp.heartbeat.Heartbeat
     :param int channel_id: The channel number to send the frame on
     :rtype: str
     :raises: ValueError
@@ -93,11 +108,11 @@ def marshal(frame_value, channel_id):
     raise ValueError('Could not determine frame type: %r', frame_value)
 
 
-def _demarshal_protocol_header_frame(data_in):
-    """Attempt to demarshal a protocol header frame
+def _unmarshal_protocol_header_frame(data_in):
+    """Attempt to unmarshal a protocol header frame
 
     The ProtocolHeader is abbreviated in size and functionality compared to
-    the rest of the frame types, so return DEMARSHALLING_ERROR doesn't apply
+    the rest of the frame types, so return UNMARSHAL_ERROR doesn't apply
     as cleanly since we don't have all of the attributes to return even
     regardless of success or failure.
 
@@ -107,17 +122,18 @@ def _demarshal_protocol_header_frame(data_in):
 
     """
     # Do the first four bytes match?
-    if data_in[0:4] == 'AMQP':
+
+    if data_in[0:4] == AMQP:
         try:
             frame = header.ProtocolHeader()
-            frame.demarshal(data_in)
+            frame.unmarshal(data_in)
             return frame
         except IndexError:
             raise ValueError('Frame data did not meet minimum length')
 
 
-def _demarshal_method_frame(frame_data):
-    """Attempt to demarshal a method frame
+def _unmarshal_method_frame(frame_data):
+    """Attempt to unmarshal a method frame
 
     :param str frame_data: Raw frame data to assign to our method frame
     :return tuple: Amount of data consumed and the frame object
@@ -126,24 +142,29 @@ def _demarshal_method_frame(frame_data):
     # Get the Method Index from the class data
     bytes_used, method_index = codec.decode.long_int(frame_data[0:4])
 
-    # Create an instance of the method object we're going to demarshal
+    print(bytes_used)
+    print(method_index)
+    print(specification.INDEX_MAPPING[method_index])
+
+    # Create an instance of the method object we're going to unmarshal
     try:
         method = specification.INDEX_MAPPING[method_index]()
     except KeyError as error:
-        raise exceptions.DemarshalingException('Unknown', error)
+        raise exceptions.UnmarshalingException('Unknown', error)
 
-    # Demarshal the data
+    # Unmarshal the data
     try:
-        method.demarshal(frame_data[bytes_used:])
+        print(frame_data[bytes_used:])
+        method.unmarshal(frame_data[bytes_used:])
     except struct.error as error:
-        raise exceptions.DemarshalingException(method, error)
+        raise exceptions.UnmarshalingException(method, error)
 
-    #  Demarshal the data in the object and return it
+    #  Unmarshal the data in the object and return it
     return method
 
 
-def _demarshal_header_frame(frame_data):
-    """Attempt to demarshal a header frame
+def _unmarshal_header_frame(frame_data):
+    """Attempt to unmarshal a header frame
 
     :param str frame_data: Raw frame data to assign to our header frame
     :return tuple: Amount of data consumed and the frame object
@@ -151,21 +172,21 @@ def _demarshal_header_frame(frame_data):
     """
     content_header = header.ContentHeader()
     try:
-        content_header.demarshal(frame_data)
+        content_header.unmarshal(frame_data)
     except struct.error as error:
-        raise exceptions.DemarshalingException('ContentHeader', error)
+        raise exceptions.UnmarshalingException('ContentHeader', error)
     return content_header
 
 
-def _demarshal_body_frame(frame_data):
-    """Attempt to demarshal a body frame
+def _unmarshal_body_frame(frame_data):
+    """Attempt to unmarshal a body frame
 
     :param str frame_data: Raw frame data to assign to our body frame
     :return tuple: Amount of data consumed and the frame object
 
     """
     content_body = body.ContentBody()
-    content_body.demarshal(frame_data)
+    content_body.unmarshal(frame_data)
     return content_body
 
 
@@ -181,7 +202,7 @@ def _frame_parts(data_in):
         return struct.unpack('>BHI', data_in[0:FRAME_HEADER_SIZE])
     except struct.error:
         # We didn't get a full frame
-        return DEMARSHALLING_FAILURE
+        return UNMARSHAL_FAILURE
 
 
 def _marshal(frame_type, channel_id, payload):
@@ -189,12 +210,14 @@ def _marshal(frame_type, channel_id, payload):
 
     :param int frame_type: The frame type to marshal
     :param int channel_id: The channel it will be sent on
-    :param str payload: The frame payload
-    :rtype: str
+    :param str|bytes payload: The frame payload
+    :rtype: str or bytes
 
     """
-    return (struct.pack('>BHI', frame_type, channel_id, len(payload)) +
-            payload + FRAME_END_CHAR)
+    header = struct.pack('>BHI', frame_type, channel_id, len(payload))
+    if PYTHON3:
+        return b''.join([header, payload, FRAME_END_CHAR])
+    return ''.join([header, payload, FRAME_END_CHAR])
 
 
 def _marshal_content_body_frame(frame_value, channel_id):
